#
# command line and appends
#

=SH
$sed --help >tmp.a 2>tmp.b
status=$?
if test -s tmp.b
then
  echo ---- test failed line $line: sed --help printing on stderr.
  exit 1
fi
if test $status -ne 0 
then
  echo ---- test failed line $line: sed --help with non-zero exit status.
  false
fi
.

=SH 
# get the version 
sed -f version.sed changelog.txt | sed '$!d;s/.*"\(.*\)".*/\1/' > tmp.a
$sed --version > tmp.b 2> tmp.c
status=$?
if cmp -s tmp.a tmp.b
then
  :
else
  echo ---- test failed line $line: sed --version outputs
  cat tmp.b
  echo -- instead of
  cat tmp.a
  echo --
  exit 1
fi
if test -s tmp.c
then
  echo ---- test failed line $line: sed --version printing on stderr.
  exit 1
fi
if test $status -ne 0 
then
  echo ---- test failed line $line: sed --version with non-zero exit status.
  false
fi
.


=CMD
sh -f tmp.IN >out.tmp
if cmp -s out.tmp tmp.OUT
then
  :
else
  echo ---- test failed line $line: executing  
  cat tmp.IN
  echo -- gave output
  cat out.tmp
  echo -- instead of intended
  cat tmp.OUT
  echo --
  false
fi
.

=ECMD
if sh -f tmp.IN 2>/dev/null >/dev/null
then
  echo ---- test failed line $line: executing  
  cat tmp.IN
  echo -- should provoke an error.
  false
else
  true
fi
.

=FOO s/foo/bar/

=IN echo foo | $sed -es/foo/bar/
=OUT bar
=SH sh -f tmp.CMD

=IN echo foo | $sed -e s/foo/bar/
=OUT bar
=SH sh -f tmp.CMD

=IN echo foo | $sed -ftmp.FOO
=OUT bar
=SH sh -f tmp.CMD

=IN echo foo | $sed -e
=SH sh -f tmp.ECMD

=IN echo foo | $sed -f
=SH sh -f tmp.ECMD

=IN echo foo | $sed -bad
=SH sh -f tmp.ECMD

=IN echo foo | $sed -nbad
=SH sh -f tmp.ECMD

=IN echo foo | $sed 
=SH sh -f tmp.ECMD

=IN $sed -f nonexistingfile
=SH sh -f tmp.ECMD

=IN 
echo foo | $sed -e 's/foo/\
bar/'
=OUT 

bar
=SH sh -f tmp.CMD


=BARCMD
sh -f tmp.IN
if cmp -s tmp.BAR tmp.OUT
then
  :
else
  echo ---- test failed line $line: after executing
  cat tmp.IN
  echo -- file tmp.BAR contains
  cat tmp.BAR
  echo -- instead of 
  cat tmp.OUT
  echo --
  false
fi
.

=IN echo foo | $sed -n 's/foo/bar/w tmp.BAR'
=OUT bar
=SH sh -f tmp.BARCMD


=IN echo foo | $sed -n 's/foo/bar/;w tmp.BAR'
=OUT bar
=SH sh -f tmp.BARCMD

#
# testing a big number of appends
#


=SH
echo bar > tmp.bar
# put 1000 chars in buffer
echo 0123456789 | $sed '
  s/./0123456789/g
  s/./abaababbab/g
' > tmp.in
$sed '
  s/a/foo\
/g
  s/b/bar\
/g
  s/\n$//
' tmp.in > tmp.a
$sed '
  :loop
  /^a/a\
foo
  /^b/r tmp.bar
  s/.//
  t loop
  d
' tmp.in > tmp.b
if cmp -s tmp.a tmp.b
then
  :
else
  echo "---- test failed line $line: (big number of appends)"
  echo "-- diff a b | head -20 yields:"
  diff tmp.a tmp.b | head -20
  echo --
  false
fi
.

#
# test n and N overlapping on two files
#

=ONE one
=TWO two

=IN $sed '2d;n;l;d' tmp.ONE tmp.TWO
=OUT 
one
two$
=SH sh -f tmp.CMD

=IN 
$sed '
1a\
append
n' tmp.ONE tmp.TWO
=OUT
one
append
two
=SH sh -f tmp.CMD

=IN 
$sed '
1a\
append
N' tmp.ONE tmp.TWO
=OUT
append
one
two
=SH sh -f tmp.CMD


=IN $sed '2d;N;l;d' tmp.ONE tmp.TWO
=OUT 
one
two$
=SH sh -f tmp.CMD

=IN $sed '$s/^/end: /' tmp.ONE tmp.TWO
=OUT 
one
end: two
=SH sh -f tmp.CMD

#
# n, N at last line
#

=IN foo
=OUT foo
=SED n;l
=SED p;N;l
=SED p;d
=SED p;D

#
# check that D does not increase the line number, and does not quit
# when last line is reached
#

=IN echo foo | $sed '=;h;G;P;D' | head -6
=OUT
1
foo
1
foo
1
foo
=SH sh -f tmp.CMD

#
# check that appends are output after 'q'
#

=IN foo
=OUT
foo
bar
=SED 
a\
bar
q
.

#
# test 'w' command
#

=SH
echo 'one
two' | $sed -n '
  1w tmp.a
  2s/two/bar/w tmp.b
'
read a <tmp.a
read b <tmp.b
if test "x$a" != "xone" -o "x$b" != "xbar"
then
  echo ---- "test failed line $line: ('w' command)"
  exit 1
else
  :
fi
.
